# -*- coding: utf-8 -*-

"""
    Classe para controle do leitor biometrico. Esta classe executa o programa em C++ que
    executa a lib do leitor biometrico para a comunicacao com o dispositivo.
"""

import os
from datetime import datetime
import base64


class Fingertech(object):

    def __init__(self):
        # diretorio do executavel C++
        self.arq_cpp = '/usr/bin/./fingertech '
        # diretorio e nome do arquivo em que o programa C++ vai gravar a leitura da biometria
        self.arq_biometria = '/tmp/%s-fir.txt' % str(datetime.now()).replace(' ','_')
        # diretorio e nome do arquivo em que o programa C++ vai gravar o log das
        # cominicacoes do programa C++ com o dispositivo
        self.arq_log = '/tmp/%s-ctp.txt' % str(datetime.now()).replace(' ','-')
        # guarda a string da biometria lida pelo dispositivo(Fingerprint Identification Record-FIR)
        self.fir = False
        self.fir_antigo = False
        # tempo em milisegundos que o leitor espere para posicionar o dedo para leitura
        self.timeout = 10000
        # diretorio onde os arquivos de digitais comparados por BD virtual serao salvos
        self.diretorio_firs = '/tmp/'
        # quantidade de digitais a serem comaparadas
        self.qtd_fir = 0
        # dicionario contendo id do o como chave e o fir como valor(o id pode ser qualquer numero
        # inteiro positivo)
        self.dict_firs = {}

    def set_default(self):
        self.fir = False
        self.dict_firs = {}
        self.qtd_fir = 0


    def ler_biometria(self):
        """
            Le a biometria pelo dispositivo, e grava no arquivo informado em 'arq_biometria'
        """
        try:
            # 1 = parametro para o C++ fazer a leitura
            if os.system('ls %s' % self.arq_cpp) != 0:
                raise Exception("Arquivo nao encontrado: %s " % self.arq_cpp)

            os.system(self.arq_cpp + '1 %s %s %s' % (self.timeout,
                                                     self.arq_biometria, self.arq_log))

            log = open(self.arq_log, 'r')
            lines = log.readlines()
            log.close()
            biometria = open(self.arq_biometria, 'r')
            if self.fir:
                self.fir_antigo = self.fir

            self.fir = base64.encodestring(biometria.read())
            biometria.close()
            os.system('rm -rf %s' % self.arq_log)
        except Exception as e:
            raise Exception('Erro ao gravar a biometria: ' + str(e))

        for line in lines:
            if line[0] != '3':
                raise Exception('Erro ao registrar digital: \n%s\n\nTente novamente!' % line)

        return True

    def compara_biometria(self):
        """
            Compara a biometria que sera lida no dispositivo com a biometria gravada no arquivo
            contido em 'arq_biometria'
        """
        try:
            # 2 = parametro para o C++ fazer a leitura
            os.system(self.arq_cpp + '2 %s %s %s' % (self.timeout,
                                                     self.arq_biometria, self.arq_log))
            log = open(self.arq_log, 'r')

            lines = log.readlines()
            log.close()
            os.system('rm -rf %s' % self.arq_log)
            for line in lines:
                if line[0] != '3':
                    # caso retorne erro ao verificar a biometria
                    return False

            biometria = open(self.arq_biometria, 'r')
            self.fir = base64.encodestring(biometria.read())
            biometria.close()
            os.system('rm -rf %s' % self.arq_biometria)
        except Exception as e:
            raise Exception('Erro ao comparar a biometria: ' + str(e))

        return True

    def verifica_biometria_bd(self):
        """
            Verifica em varias biometrias se a passada no parametro 'self.fir' está contida
            retornando o id do fir
        """
        res = False
        if not self.dict_firs:
            return res

        try:
            dir = '%s%s-fir-bd.txt' % (self.diretorio_firs,
                                       str(datetime.now()).replace(' ','_'))
            arq = open(dir, 'w')
            for chave in self.dict_firs:
                txt = "%03d;%s\n" % (chave, base64.decodestring(self.dict_firs.get(chave)))
                arq.write(txt)

            arq.close()
            # 5 = parametro para o C++ insere no BD as biometrias gravadas no arquivo
            # 'diretorio_firs' e compara com a biometria lida
            os.system(self.arq_cpp + '5 %s %s %s' % (self.fir, dir,
                                                        self.arq_log))
            log = open(self.arq_log, 'r')
            lines = log.readlines()
            log.close()
            os.system('rm -rf %s' % self.arq_log)
            for line in lines:
                if line[0] != '3':
                    # caso retorne erro ao verificar a biometria
                    return res
                else:
                    res = int(line[2:])

            os.system('rm -rf %s' % self.arq_biometria)
        except Exception as e:
            raise Exception('Erro verificar biometria no BD: ' + str(e))
        finally:
            self.set_default()

        return res

    def verifica_biometria(self):
        """
            Verifica se duas biometrias sao a mesma
        """
        try:
            # 3 = parametro para o C++ apenas comparacao entre duas biometrias
            # ja lidas anteriormente
            os.system(self.arq_cpp + '3 %s %s %s' % (self.fir_antigo, self.fir, self.arq_log))
            log = open(self.arq_log, 'r')
            lines = log.readlines()
            log.close()
            os.system('rm -rf %s' % self.arq_log)
            for line in lines:
                if line[0] != '3':
                    # caso retorne erro ao verificar a biometria
                    return False

            os.system('rm -rf %s' % self.arq_biometria)
        except Exception as e:
            raise Exception('Erro ao comparar a biometria: ' + str(e))

        return True

    def checa_sensor_biometrico(self):
        """
            Checa se o dedo esta na tela do leitor, retorna True se estiver e false se nao estiver
            Obs: depende de versao de driver do leitor para funcinar
        """
        try:
            os.system(self.arq_cpp + '4 %s' % (self.arq_log))
            log = open(self.arq_log, 'r')
            lines = log.readlines()
            log.close()
            os.system('rm -rf %s' % self.arq_log)
            for line in lines:
                if line[0] != '3':
                    return False

        except Exception as e:
            raise Exception('Erro ao checar sensor:%s' % str(e))

        return True

