# -*- encoding: utf-8 -*-
__Autor__ = "Luiz Fernando"
__Empresa__ = "Sig Informatica Ltda"
__Site__ = "http://www.siginformatica.com.br"

import sys
import re

if __name__ == '__main__':
    try:
        if sys.argv[1] == '-9':
            print('inscricao_estadual.py\t\t\tValidador de IE')
        else:
            print('\n\ninscricao_estadual.py - Validador de IE')
                   
    except:
        print('\n\ninscricao_estadual.py - Validador de IE')
        
def validar(estado, jur_ie):
        """Checks if company register number in field insc_est is valid, 
        this method call others methods because this validation is State wise
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of partner Ids,
        @return: True or False.
        """
        validate = '_validate_ie_%s' % estado.lower()

        validate = eval(validate)
        if callable(validate): 
            return validate(jur_ie)

        return False

def _validate_ie_ac(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Acre
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 13:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '01':
        return False

    # Pega apenas os 11 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:11]

    prod = [4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
    while len(nova_ie) < 13:
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r > 1:
            f = 11 - r
        else:
            f = 0
        nova_ie.append(f)
        prod.insert(0, 5)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2]+ '.' + nova_ie[2:4]+ '.' + nova_ie[4:8]+ '-' + nova_ie[8:]

    return False

def _validate_ie_al(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Alagoas
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '24':
        return False

    # verificando o tipo de empresa
    #if not inscr_est[2] in ['0', '3', '5', '7', '8']:
    #    return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = (sum([x*y for (x, y) in zip(nova_ie, prod)]) * 10) % 11
    if r > 9:
        r = 0
    nova_ie.append(r)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2]+ '.' + nova_ie[2:4]+ '.' + nova_ie[4:8]+ '-' + nova_ie[8:]

    return False

def _validate_ie_am(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Amazonas
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2]+ '.' + nova_ie[2:4]+ '.' + nova_ie[4:8]+ '-' + nova_ie[8:]

    return False

def _validate_ie_ap(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Amapá
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '03':
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # define os valores de 'p' e 'd'
    inscr_est_int = int(inscr_est[:8])
    if (inscr_est_int <= 3017000):
        inscr_est_p = 5
        inscr_est_d = 0
    if (inscr_est_int >= 3017001) and (inscr_est_int <= 3019022):
        inscr_est_p = 9
        inscr_est_d = 1
    if (inscr_est_int >= 3019023):
        inscr_est_p = 0
        inscr_est_d = 0

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = (inscr_est_p + sum([x*y for (x, y) in zip(nova_ie, prod)])) % 11
    if r > 1:
        f = 11 - r
    elif r == 1:
        f = 0
    else:
        f = inscr_est_d
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2]+ '.' + nova_ie[2:4]+ '.' + nova_ie[4:8]+ '-' + nova_ie[8:]

    return False

def _validate_ie_ba(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Bahia
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if (len(inscr_est) != 8) and (len(inscr_est) != 9):
        return False

    inscr_est = list(map(int, inscr_est))

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 8:

        # Pega apenas os 6 primeiros dígitos da inscrição estadual e
        # gera os dígitos verificadores
        nova_ie = inscr_est[:6]

        prod = [7, 6, 5, 4, 3, 2]

        if inscr_est[0] in [0, 1, 2, 3, 4, 5, 8]:
            modulo = 10
        else:
            modulo = 11

        while len(nova_ie) < 8:

            r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % modulo
            if r > 0:
                f = modulo - r
            else:
                f = 0

            if len(nova_ie) < 7:
                nova_ie.append(f)
            else:
                nova_ie.insert(6, f)

            prod.insert(0, 8)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 9:

        # Pega apenas os 7 primeiros dígitos da inscrição estadual e
        # gera os dígitos verificadores
        nova_ie = inscr_est[:7]

        prod = [8, 7, 6, 5, 4, 3, 2]

        if inscr_est[1] in [0, 1, 2, 3, 4, 5, 8]:
            modulo = 10
        else:
            modulo = 11

        while len(nova_ie) < 9:

            r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % modulo
            if r > 0:
                f = modulo - r
            else:
                f = 0

            if len(nova_ie) < 8:
                nova_ie.append(f)
            else:
                nova_ie.insert(7, f)

            prod.insert(0, 9)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:3]+ '.' + nova_ie[3:6]+ '-' + nova_ie[6:]

    return False

def _validate_ie_ce(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Ceará
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2]+ '.' + nova_ie[2:4]+ '.' + nova_ie[4:8]+ '-' + nova_ie[8:]

    return False

def _validate_ie_df(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Distitro Federal
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False._check_ie
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 13:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '07':
        return False

    # Pega apenas os 11 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:11]

    prod = [4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
    while len(nova_ie) < 13:
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r > 1:
            f = 11 - r
        else:
            f = 0
        nova_ie.append(f)
        prod.insert(0, 5)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:11] + '-' + nova_ie[11:]

    return False

def _validate_ie_es(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Espirito Santo
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:3] + '.' + nova_ie[3:6] + '.' + nova_ie[6:8]+ '-' + nova_ie[8:]

    return False
 
def _validate_ie_go(inscr_est):
    """Checks if company register number is val_check_ieid to Brazilian
    state Goiais
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if not inscr_est[:2] in ['10', '11', '15']:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # define os valores de 'p' e 'd'
    inscr_est_int = int(inscr_est[:8])
    if ( inscr_est_int >= 10103105 ) and ( inscr_est_int <= 10119997 ):
        inscr_est_d = 1
    else:
        inscr_est_d = 0

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    elif r == 1:
        f = inscr_est_d
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:5] + '.' + nova_ie[5:8]+ '-' + nova_ie[8:]

    return False
  
def _validate_ie_ma(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Maranhão
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '12':
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_mg(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Minas Gerais
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 13:
        return False

    # Pega apenas os 11 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:11]

    nova_ie_aux = list(nova_ie)
    prod = [1, 2, 1, 1, 2, 1, 2, 1, 2, 1, 2]
    r = str([x*y for (x, y) in zip(nova_ie_aux, prod)])
    r = re.sub('[^0-9]', '', r)
    r = list(map(int, r))
    r = sum(r)
    r2 = (int(r/10)+1)*10
    r = r2 - r
    r = list(map(int, str(r)))[-1:][0]

    nova_ie.append(r)

    prod = [3, 2, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:9]+ '-' + nova_ie[9:]

    return False
  
def _validate_ie_ms(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Mato Grosso do Sul
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '28':
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:5] + '.' + nova_ie[5:8] + '-' + nova_ie[8:]

    return False

def _validate_ie_mt(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Mato Grosso
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 11:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:10]

    prod = [3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:10] + '-' + nova_ie[10:]

    return False
 
def _validate_ie_pa(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Pará
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '15':
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_pb(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Paraíba
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_pe(inscr_est):
    """Check if number in insc_est is valid to Brazilian
    state of Pernambuco
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if (len(inscr_est) != 9) and (len(inscr_est) != 14):
        return False

    inscr_est = list(map(int, inscr_est))

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 9:

        # Pega apenas os 7 primeiros dígitos da inscrição estadual e
        # gera os dígitos verificadores
        inscr_est = map(int, inscr_est)
        nova_ie = inscr_est[:7]

        prod = [8, 7, 6, 5, 4, 3, 2]
        while len(nova_ie) < 9:
            r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
            if r > 1:
                f = 11 - r
            else:
                f = 0
            nova_ie.append(f)
            prod.insert(0, 9)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 14:

        # Pega apenas os 13 primeiros dígitos da inscrição estadual e
        # gera o dígito verificador
        inscr_est = list(map(int, inscr_est))
        nova_ie = inscr_est[:13]

        prod = [5, 4, 3, 2, 1, 9, 8, 7, 6, 5, 4, 3, 2]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        f = 11 - r
        if f > 10:
            f = f - 10
        nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:7] + '-' + nova_ie[7:]

    return False
  
def _validate_ie_pi(inscr_est):
    """Check if number in insc_est is valid to Brazilian
    state of Piauí
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_pr(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Rio de Paraná
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 10:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [3, 2, 7, 6, 5, 4, 3, 2]
    while len(nova_ie) < 10:
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r > 1:
            f = 11 - r
        else:
            f = 0
        nova_ie.append(f)
        prod.insert(0, 4)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_rj(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Rio de janeiro
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """
    
    # Limpando a inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 8:
        return False

    # Pega apenas os 12 primeiros dígitos da inscrição estadual e
    # gera os dígitos verificadores
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:7]

    prod = [2, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:5] + '.' + nova_ie[5:]

    return False

def _validate_ie_rn(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Rio Grande do Norte
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if (len(inscr_est) != 9) and (len(inscr_est) != 10):
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '20':
        return False

    inscr_est = list(map(int, inscr_est))

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 9:

        # Pega apenas os 8 primeiros dígitos da inscrição estadual e
        # gera o dígito verificador
        nova_ie = inscr_est[:8]

        prod = [9, 8, 7, 6, 5, 4, 3, 2]
        r = (sum([x*y for (x, y) in zip(nova_ie, prod)]) * 10) % 11
        if r > 9:
            r = 0
        nova_ie.append(r)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 10:

        # Pega apenas os 9 primeiros dígitos da inscrição estadual e
        # gera o dígito verificador
        nova_ie = inscr_est[:9]

        prod = [10, 9, 8, 7, 6, 5, 4, 3, 2]
        r = (sum([x*y for (x, y) in zip(nova_ie, prod)]) * 10) % 11
        if r > 9:
            r = 0
        nova_ie.append(r)


    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:5] + '.' + nova_ie[5:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_ro(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Rondônia
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if (len(inscr_est) != 9) and (len(inscr_est) != 14):
        return False

    inscr_est = list(map(int, inscr_est))

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 9:

        # Despreza-se os 3 primeiros dígitos, pega apenas os 8 primeiros
        # dígitos da inscrição estadual e gera o dígito verificador
        nova_ie = inscr_est[3:8]

        prod = [6, 5, 4, 3, 2]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        f = 11 - r
        if f > 9:
            f = f - 10
        nova_ie.append(f)

        nova_ie.insert(0, inscr_est[0])
        nova_ie.insert(1, inscr_est[1])
        nova_ie.insert(2, inscr_est[2])

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) == 14:

        # Pega apenas os 13 primeiros dígitos da inscrição estadual e
        # gera o dígito verificador
        nova_ie = inscr_est[:13]

        prod = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        f = 11 - r
        if f > 9:
            f = f - 10
        nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        if nova_ie[:3] == '000':
           return nova_ie[:13] + '-' + nova_ie[13:]
        else:
           return nova_ie[:3] + '.' + nova_ie[3:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_rr(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Roraima
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando a inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # verificando os dois primeiros dígitos
    if inscr_est[:2] != '24':
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [1, 2, 3, 4, 5, 6, 7, 8]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 9
    nova_ie.append(r)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_rs(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Rio Grande do Sul
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 10:
        return False

    # Pega apenas os 9 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:9]

    prod = [2, 9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:3] + '/' + nova_ie[3:10]

    return False

def _validate_ie_sc(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Santa Catarina
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:3] + '.' + nova_ie[3:6] + '.' + nova_ie[6:]

    return False
 
def _validate_ie_se(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Sergipe
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 9:
        return False

    # Pega apenas os 8 primeiros dígitos da inscrição estadual e
    # gera o dígito verificador
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:8]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:8] + '-' + nova_ie[8:]

    return False
 
def _validate_ie_sp(inscr_est):
    """Checks if company register number is valid to Brazilian
    state São Paulo
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Industriais e comerciais
    if inscr_est[0] != 'P':

        # Limpando o campo inscrição estadual
        if not inscr_est.isdigit():
            inscr_est = re.sub('[^0-9]', '', inscr_est)

        # verificando o tamanho da inscrição estadual
        if len(inscr_est) != 12:
            return False

        # Pega apenas os 8 primeiros dígitos da inscrição estadual e
        # gera o primeiro dígito verificador
        inscr_est = list(map(int, inscr_est))
        nova_ie = inscr_est[:8]

        prod = [1, 3, 4, 5, 6, 7, 8, 10]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r < 10:
            f = r
        if r == 10:
            f = 0
        if r == 11:
            f = 1
        nova_ie.append(f)

        # gera o segundo dígito verificador
        nova_ie.append(inscr_est[9])
        nova_ie.append(inscr_est[10])
        prod = [3, 2, 10, 9, 8, 7, 6, 5, 4, 3, 2]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r < 10:
            f = r
        if r == 10:
            f = 0
        if r == 11:
            f = 1
        nova_ie.append(f)

    # Produtor rural
    else:

        # Limpando o campo inscrição estadual
        if not inscr_est.isdigit():
            inscr_est = re.sub('[^0-9]', '', inscr_est)

        # verificando o tamanho da inscrição estadual
        if len(inscr_est) != 12:
            return False

        # verificando o primeiro dígito depois do 'P'
        if inscr_est[0] != '0':
            return False

        # Pega apenas os 8 primeiros dígitos da inscrição estadual e
        # gera o dígito verificador
        inscr_est = list(map(int, inscr_est))
        nova_ie = inscr_est[:8]

        prod = [1, 3, 4, 5, 6, 7, 8, 10]
        r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
        if r < 10:
            f = r
        if r == 10:
            f = 0
        if r == 11:
            f = 1
        nova_ie.append(f)

        nova_ie.append(inscr_est[9])
        nova_ie.append(inscr_est[10])
        nova_ie.append(inscr_est[11])

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        if nova_ie[0] == 'P':
           return nova_ie[:1] + '-' + nova_ie[1:13]
        else:
           return nova_ie[:3] + '.' + nova_ie[3:9] + '.' + nova_ie[9:]

    return False
 
def _validate_ie_to(inscr_est):
    """Checks if company register number is valid to Brazilian
    state Tocantins
    @param self: The object pointer
    @param inscr_est: The company state number value,
    @return: True or False.
    """

    # Limpando o campo inscrição estadual
    if not inscr_est.isdigit():
        inscr_est = re.sub('[^0-9]', '', inscr_est)

    # verificando o tamanho da inscrição estadual
    if len(inscr_est) != 11:
        return False

    # verificando os dígitos 3 e 4
    if not inscr_est[2:4] in ['01', '02', '03', '99']:
        return False

    # Pega apenas os dígitos que entram no cálculo
    inscr_est = list(map(int, inscr_est))
    nova_ie = inscr_est[:2] + inscr_est[4:10]

    prod = [9, 8, 7, 6, 5, 4, 3, 2]
    r = sum([x*y for (x, y) in zip(nova_ie, prod)]) % 11
    if r > 1:
        f = 11 - r
    else:
        f = 0
    nova_ie.append(f)

    nova_ie.insert(2, inscr_est[2])
    nova_ie.insert(3, inscr_est[3])

    # Se o número gerado coincidir com o número original, é válido
    if nova_ie == inscr_est:
        nova_ie = "".join(map(str, nova_ie))
        return nova_ie[:2] + '.' + nova_ie[2:5] + '.' + nova_ie[5:8] + '-' + nova_ie[8:]

    return False
